package ch.ethz.karto.map3d;

import com.sun.opengl.impl.windows.WindowsDummyGLDrawable;
import ika.utils.Sys;
import javax.media.opengl.GL;
import javax.media.opengl.GLContext;

/**
 * Manages access to OpenGL on Windows systems without pbuffer support.
 * This is only for older hardware.
 * All methods must be called from the OpenGL thread.
 * @author Bernhard Jenny, Institute of Cartography, ETH Zurich
 */
public class OpenGLCapabilitiesWindowsNoPBuffer extends OpenGLCapabilities {

    // cache drawable and gl to release properly when not used anymore.
    private WindowsDummyGLDrawable dummyDrawable;
    private GL dummyGL;

    public OpenGLCapabilitiesWindowsNoPBuffer() {
        if (!Sys.isWindows()) {
            throw new IllegalStateException();
        }
    }

    /**
     * Overwrite with Windows specific version that does not use pbuffers.
     * @return
     */
    @Override
    public GL getGL() {
        dummyDrawable = new WindowsDummyGLDrawable();
        GLContext dummyContext = dummyDrawable.createContext(null);
        if (dummyContext != null) {
            switchToNewContext(dummyContext);
            dummyGL = dummyContext.getGL();
        }

        return dummyGL;
    }

    /**
     * Overwrite with version that releases Windows-specific resources.
     */
    @Override
    protected void restoreLastGLContext() {

        if (newContext != null) {
            newContext.release();
            newContext.destroy();
            dummyDrawable.destroy();
        }
        if (lastContext != null) {
            lastContext.makeCurrent();
        }

    }
}

