/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * Map3DLODPanel.java
 *
 * Created on Mar 9, 2010, 9:43:57 AM
 */

package ch.ethz.karto.map3d.gui;

import ch.ethz.karto.map3d.Map3DGLCapabilities;
import ika.geo.GeoGrid;
import ika.geo.grid.GaussianPyramid;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import javax.swing.JLabel;
import javax.swing.JRadioButton;

/**
 *
 * @author jenny
 */
public class Map3DLODPanel extends javax.swing.JPanel {

    /**
     * the maximum number of cells in horizontal or vertical direction that
     * are displayed.
     */
    private static final int MAX_GRID_SIDE = 10000;

    /**
     * when starting, a grid is displayed with approximately this size is
     * selected from the LOD pyramid and displayed.
     */
    private static final int DEFAULT_GRID_SIDE = 1024;

    /** Creates new form Map3DLODPanel */
    public Map3DLODPanel() {
        initComponents();
    }

    /**
     * Returns the pyramid level that should be displayed as selected in the GUI.
     * @return
     */
    public int getDisplayLevel(GaussianPyramid gridPyramid) {
        int buttonCount = lodButtonPanel.getComponentCount();
        for (int i = 0; i < buttonCount; i++) {
            JRadioButton b = (JRadioButton)lodButtonPanel.getComponent(i);
            if (b.isSelected()) {
                return gridPyramid.getLevelsCount() - i - 1;
            }
        }
        return 0;
    }

    /**
     * Returns the grid that should be displayed in the 3D view
     * @return the displayGrid
     */
    public GeoGrid getDisplayGrid(GaussianPyramid gridPyramid) {
        if (gridPyramid == null) {
            return null;
        }
        return gridPyramid.getLevel(this.getDisplayLevel(gridPyramid));
    }

    /**
     * Returns the pyramid level that is initially displayed.
     * @return
     */
    public int getDefaultDisplayLevel(GaussianPyramid gridPyramid) {

        final int defaultNodeCount;
        if (Map3DGLCapabilities.hasVBO()) {
            defaultNodeCount = DEFAULT_GRID_SIDE * DEFAULT_GRID_SIDE;
        } else if (Map3DGLCapabilities.hasOpenGLVersion(1, 5)){
            // half the size if VBOs are not supported, but OpenGL 1.5 is present
            defaultNodeCount = DEFAULT_GRID_SIDE * DEFAULT_GRID_SIDE / 4;
        } else {
            // fourth the size for systems with OpenGL older than 1.5
            defaultNodeCount = DEFAULT_GRID_SIDE * DEFAULT_GRID_SIDE / 16;
        }

        int lastLevel = gridPyramid.getLevelsCount() - 1;
        for (int i = lastLevel; i >= 0; i--) {
            GeoGrid grid = gridPyramid.getLevel(i);
            int cols = grid.getCols();
            int rows = grid.getRows();
            if (cols * rows >= defaultNodeCount) {
                return i;
            }
        }
        return 0;
    }

    /**
     * Setup the panel for selecting the level of details.
     */
    public void initLevelOfDetailGUI(GaussianPyramid gridPyramid,
            ActionListener actionListener) {

        final int maxNodeCount;
        if (Map3DGLCapabilities.hasVBO()) {
            maxNodeCount = MAX_GRID_SIDE * MAX_GRID_SIDE;
        } else if (Map3DGLCapabilities.hasOpenGLVersion(1, 5)){
            // half the size if VBOs are not supported, but OpenGL 1.5 is present
            maxNodeCount = MAX_GRID_SIDE * MAX_GRID_SIDE / 4;
        } else {
            // fourth the size for systems with OpenGL older than 1.5
            maxNodeCount = MAX_GRID_SIDE * MAX_GRID_SIDE / 16;
        }

        // find firstLevel, i.e. the ID of the largest pyramid level that can be
        // reasonable displayed
        int firstLevel, lastLevel = gridPyramid.getLevelsCount() - 1;
        for (firstLevel = 0; firstLevel <= lastLevel; firstLevel++) {
            GeoGrid grid = gridPyramid.getLevel(firstLevel);
            int cols = grid.getCols();
            int rows = grid.getRows();
            if (cols * rows <= maxNodeCount) {
                break;
            }
        }

        // find the default pyramid level to display
        int displayLevel = getDefaultDisplayLevel(gridPyramid);

        // add radio buttons to GUI
        lodButtonPanel.removeAll();
        DecimalFormat df = new DecimalFormat("#,##0");
        for (int i = lastLevel; i >= firstLevel; i--) {
            JRadioButton b = new JRadioButton();

            // generate label for radio button
            GeoGrid grid = gridPyramid.getLevel(i);
            StringBuilder sb = new StringBuilder();
            sb.append(df.format(grid.getCols()));
            sb.append("\u2006\u00D7\u2006"); // multiplication sign surrounded by small spaces
            sb.append(df.format(grid.getRows()));

            // generate tooltip for radio button
            String tooltip = "Preview with " + sb.toString() + " grid cells.";
            if (lastLevel - firstLevel == 0) {
                tooltip += " Disabled because the model is small.";
            }
            b.setToolTipText(tooltip);

            // finish label
            if (i == 0) {
                sb.append(" (full resolution)");
            } else if (i == lastLevel) {
                sb.append(" (fastest)");
            }
            b.setText(sb.toString());

            // add event listener
            b.addActionListener(actionListener);

            lodButtonGroup.add(b);
            lodButtonPanel.add(b);

            // select the button of the currend display level (after adding it
            // to the button group)
            b.setSelected(displayLevel == i);

            // disable radio button if there is only one button
            b.setEnabled(lastLevel - firstLevel >= 1);
        }

        if (firstLevel != 0) {
            StringBuilder sb = new StringBuilder();
            sb.append("Full size: ");
            GeoGrid bigGrid = gridPyramid.getFullResolutionLevel();
            sb.append(df.format(bigGrid.getCols()));
            sb.append("\u2006\u00D7\u2006"); // multiplication sign surrounded by small spaces
            sb.append(df.format(bigGrid.getRows()));
            JLabel label = new JLabel(sb.toString());
            label.setFont(label.getFont().deriveFont(11f));
            lodButtonPanel.add(label);
        }
        lodButtonPanel.revalidate();
        lodButtonPanel.repaint();

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        lodButtonGroup = new javax.swing.ButtonGroup();
        javax.swing.JPanel lodPanel_ = new TransparentMacPanel();
        javax.swing.JLabel titleLabel = new javax.swing.JLabel();
        javax.swing.JTextArea lodInfoTextArea = new javax.swing.JTextArea();
        lodButtonPanel = new TransparentMacPanel();

        lodPanel_.setLayout(new java.awt.GridBagLayout());

        titleLabel.setText("Level of Detail for Preview");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 0);
        lodPanel_.add(titleLabel, gridBagConstraints);

        lodInfoTextArea.setColumns(20);
        lodInfoTextArea.setEditable(false);
        lodInfoTextArea.setFont(new java.awt.Font("Lucida Grande", 0, 11));
        lodInfoTextArea.setLineWrap(true);
        lodInfoTextArea.setRows(2);
        lodInfoTextArea.setText("For export the full resolution is used.");
        lodInfoTextArea.setWrapStyleWord(true);
        lodInfoTextArea.setOpaque(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(20, 0, 0, 0);
        lodPanel_.add(lodInfoTextArea, gridBagConstraints);

        lodButtonPanel.setLayout(new javax.swing.BoxLayout(lodButtonPanel, javax.swing.BoxLayout.Y_AXIS));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
        lodPanel_.add(lodButtonPanel, gridBagConstraints);

        add(lodPanel_);
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup lodButtonGroup;
    private javax.swing.JPanel lodButtonPanel;
    // End of variables declaration//GEN-END:variables

}
