/*
 * LonLatPanel.java
 *
 * Created on January 30, 2008, 2:18 PM
 */

package ika.gui;

import java.awt.Component;
import java.awt.geom.Point2D;
import javax.swing.JOptionPane;

/**
 * A dialog to let the user specify a longitude and a latitude.
 * @author  Bernhard Jenny, Institute of Cartography, ETH Zurich.
 */
public class LonLatPanel extends javax.swing.JPanel {
    
    public static void main(String[] args) {
        LonLatPanel panel = new LonLatPanel();
        panel.setMessage("Toto");
        panel.setLonLat(55, 33);
        Point2D point = panel.showDialog(null, "Gino");
        System.out.println(point);
    } 
    
    /** Creates new form LonLatPanel */
    public LonLatPanel() {
        initComponents();
    }
    
    public void setMessage (String message) {
        this.messageLabel.setText(message);
    }
    
    public void setLonLat(double lon, double lat) {
        this.lonNumberField.setDoubleValue(lon);
        this.latNumberField.setDoubleValue(lat);       
    }
    
    public Point2D showDialog(Component parentComponent, String title) {
        
        int n = JOptionPane.showOptionDialog(parentComponent, this,
                title, 
                JOptionPane.PLAIN_MESSAGE, 
                JOptionPane.NO_OPTION, 
                null, 
                new Object[]{"OK", "Cancel"}, 
                null); // no default button so that the enter key confirms entries in the number fields
        if (n == 0) {
            return new Point2D.Double(lonNumberField.getDoubleValue(), 
                    latNumberField.getDoubleValue());
        } else {
            return null;
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        messageLabel = new javax.swing.JLabel();
        lonSlider = new javax.swing.JSlider();
        latSlider = new javax.swing.JSlider();
        lonNumberField = new ika.gui.NumberField();
        latNumberField = new ika.gui.NumberField();
        javax.swing.JLabel jLabel1 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel2 = new javax.swing.JLabel();

        setLayout(new java.awt.GridBagLayout());

        messageLabel.setText("Select a Point");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 20, 0);
        add(messageLabel, gridBagConstraints);

        lonSlider.setMajorTickSpacing(45);
        lonSlider.setMaximum(180);
        lonSlider.setMinimum(-180);
        lonSlider.setMinorTickSpacing(15);
        lonSlider.setPaintTicks(true);
        lonSlider.setValue(0);
        lonSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                lonSliderStateChanged(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        add(lonSlider, gridBagConstraints);

        latSlider.setMajorTickSpacing(45);
        latSlider.setMaximum(90);
        latSlider.setMinimum(-90);
        latSlider.setMinorTickSpacing(15);
        latSlider.setPaintTicks(true);
        latSlider.setValue(0);
        latSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                latSliderStateChanged(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        add(latSlider, gridBagConstraints);

        lonNumberField.setMax(180.0);
        lonNumberField.setMin(-180.0);
        lonNumberField.setPreferredSize(new java.awt.Dimension(60, 28));
        lonNumberField.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                lonNumberFieldPropertyChange(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        add(lonNumberField, gridBagConstraints);

        latNumberField.setMax(90.0);
        latNumberField.setMin(-90.0);
        latNumberField.setPreferredSize(new java.awt.Dimension(60, 28));
        latNumberField.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                latNumberFieldPropertyChange(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        add(latNumberField, gridBagConstraints);

        jLabel1.setText("Longitude:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel1, gridBagConstraints);

        jLabel2.setText("Latitude:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel2, gridBagConstraints);
    }// </editor-fold>//GEN-END:initComponents

    private void lonSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_lonSliderStateChanged
        lonNumberField.setDoubleValue(lonSlider.getValue());
}//GEN-LAST:event_lonSliderStateChanged

    private void latSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_latSliderStateChanged
        latNumberField.setDoubleValue(latSlider.getValue());
    }//GEN-LAST:event_latSliderStateChanged

    private void lonNumberFieldPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_lonNumberFieldPropertyChange
        if ("value".equals(evt.getPropertyName())) {
            lonSlider.setValue((int)lonNumberField.getDoubleValue());
        }
    }//GEN-LAST:event_lonNumberFieldPropertyChange

    private void latNumberFieldPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_latNumberFieldPropertyChange
        if ("value".equals(evt.getPropertyName())) {
            latSlider.setValue((int)latNumberField.getDoubleValue());
        }
    }//GEN-LAST:event_latNumberFieldPropertyChange
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private ika.gui.NumberField latNumberField;
    private javax.swing.JSlider latSlider;
    private ika.gui.NumberField lonNumberField;
    private javax.swing.JSlider lonSlider;
    private javax.swing.JLabel messageLabel;
    // End of variables declaration//GEN-END:variables
    
}
