/*
 * Worker.java
 *
 * Created on February 1, 2007, 9:20 PM
 *
 */

package ika.gui;

/**
 *
 * @author Bernhard Jenny, Institute of Cartography, ETH Zurich
 */
abstract public class Worker {
    
    /**
     * A ProgressIndicator that is displayed while work() is working.
     * The ProgressIndicator must be set using setProgressIndicator().
     * The default is not to have any ProgressIndicator.
     */
    protected ProgressIndicator progressIndicator = null;
    
    protected WorkerParameters params = null;
    
    /** Creates a new instance of Worker */
    public Worker() {
    }
    
    /**
     * work() does all the work. It is responsible for periodically calling
     * progressIndicator.progress(int percentage)
     */
    abstract protected void work();
    
        public void setProgressIndicator(ProgressIndicator progressIndicator) {
        this.progressIndicator = progressIndicator;
    }

    /**
     * Read data.
     */
    public void run() {

        try {
            this.work();
        } catch (Exception e) {
            // hide the progress dialog before informing about the exception.
            if (this.progressIndicator != null)
                this.progressIndicator.complete();
            this.progressIndicator = null;
            
            // pass the exception to the WorkerParameters.
            // It is responsible for informing the user.
            params.error(e);
        } finally {
            if (this.progressIndicator != null)
                this.progressIndicator.complete();
        }
        
    }
}
